@@ -1,24 +1,24 @@
 from __future__ import annotations
-import random
-from typing import Any, AsyncGenerator, Dict, List, Optional
 
+import asyncio
+import random
+from typing import Any, AsyncGenerator, Dict, Optional
+
+from notionary import NotionPage
 from notionary.database.client import NotionDatabaseClient
+from notionary.database.database_filter_builder import DatabaseFilterBuilder
+from notionary.database.database_provider import NotionDatabaseProvider
 from notionary.models.notion_database_response import (
     NotionDatabaseResponse,
     NotionPageResponse,
     NotionQueryDatabaseResponse,
 )
-from notionary.page.notion_page import NotionPage
-
-from notionary.database.notion_database_provider import NotionDatabaseProvider
-
-from notionary.database.filter_builder import FilterBuilder
 from notionary.telemetry import (
-    ProductTelemetry,
     DatabaseFactoryUsedEvent,
+    ProductTelemetry,
     QueryOperationEvent,
 )
-from notionary.util import factory_only, LoggingMixin
+from notionary.util import LoggingMixin, factory_only
 
 
 class NotionDatabase(LoggingMixin):
@@ -27,7 +27,7 @@ class NotionDatabase(LoggingMixin):
     Focused exclusively on creating basic pages and retrieving page managers
     for further page operations.
     """
-    
+
     telemetry = ProductTelemetry()
 
     @factory_only("from_database_id", "from_database_name")
@@ -213,7 +213,7 @@ class NotionDatabase(LoggingMixin):
             self.logger.error(f"Error updating database external icon: {str(e)}")
             return None
 
-    async def get_options_by_property_name(self, property_name: str) -> List[str]:
+    async def get_options_by_property_name(self, property_name: str) -> list[str]:
         """
         Retrieve all option names for a select, multi_select, status, or relation property.
 
@@ -244,7 +244,7 @@ class NotionDatabase(LoggingMixin):
         property_schema = self.properties.get(property_name)
         return property_schema.get("type") if property_schema else None
 
-    async def query_database_by_title(self, page_title: str) -> List[NotionPage]:
+    async def query_database_by_title(self, page_title: str) -> list[NotionPage]:
         """
         Query the database for pages with a specific title.
         """
@@ -254,7 +254,7 @@ class NotionDatabase(LoggingMixin):
             )
         )
 
-        page_results: List[NotionPage] = []
+        page_results: list[NotionPage] = []
 
         for page in search_results.results:
             page = await NotionPage.from_page_id(
@@ -272,10 +272,10 @@ class NotionDatabase(LoggingMixin):
         self, hours: int = 24, page_size: int = 100
     ) -> AsyncGenerator[NotionPage, None]:
         """
-        Iterate through pages edited in the last N hours using FilterBuilder.
+        Iterate through pages edited in the last N hours using DatabaseFilterBuilder.
         """
 
-        filter_builder = FilterBuilder()
+        filter_builder = DatabaseFilterBuilder()
         filter_builder.with_updated_last_n_hours(hours).with_page_size(page_size)
         filter_conditions = filter_builder.build()
 
@@ -284,13 +284,39 @@ class NotionDatabase(LoggingMixin):
         ):
             yield page
 
-    async def get_all_pages(self) -> List[NotionPage]:
+    async def get_all_pages(self) -> list[NotionPage]:
         """
         Get all pages in the database (use with caution for large databases).
+        Uses asyncio.gather to parallelize NotionPage creation per API page.
         """
-        pages = []
-        async for page in self._iter_pages():
-            pages.append(page)
+        pages: list[NotionPage] = []
+        start_cursor: Optional[str] = None
+        has_more = True
+        body: Dict[str, Any] = {"page_size": 100}
+
+        while has_more:
+            current_body = body.copy()
+            if start_cursor:
+                current_body["start_cursor"] = start_cursor
+
+            result = await self.client.query_database(
+                database_id=self.id, query_data=current_body
+            )
+
+            if not result or not result.results:
+                break
+
+            # Parallelize NotionPage creation for this batch
+            page_tasks = [
+                NotionPage.from_page_id(page_id=page.id, token=self.client.token)
+                for page in result.results
+            ]
+            batch_pages = await asyncio.gather(*page_tasks)
+            pages.extend(batch_pages)
+
+            has_more = result.has_more
+            start_cursor = result.next_cursor if has_more else None
+
         return pages
 
     async def get_last_edited_time(self) -> Optional[str]:
@@ -313,12 +339,12 @@ class NotionDatabase(LoggingMixin):
             )
             return None
 
-    def create_filter(self) -> FilterBuilder:
+    def create_filter(self) -> DatabaseFilterBuilder:
         """Create a new filter builder for this database."""
-        return FilterBuilder()
+        return DatabaseFilterBuilder()
 
     async def iter_pages_with_filter(
-        self, filter_builder: FilterBuilder, page_size: int = 100
+        self, filter_builder: DatabaseFilterBuilder, page_size: int = 100
     ):
         """Iterate pages using a filter builder."""
         filter_config = filter_builder.build()
@@ -440,7 +466,7 @@ class NotionDatabase(LoggingMixin):
         except (KeyError, TypeError, AttributeError):
             return None
 
-    async def _get_relation_options(self, property_name: str) -> List[Dict[str, Any]]:
+    async def _get_relation_options(self, property_name: str) -> list[Dict[str, Any]]:
         """
         Retrieve the titles of all pages related to a relation property.
 
