@@ -1,21 +1,29 @@
 from __future__ import annotations
+
 import asyncio
-from typing import Any, Dict, List, Optional, TYPE_CHECKING
 import random
+from typing import TYPE_CHECKING, Any, Callable, Optional, Union
 
-from notionary.blocks import BlockRegistry, BlockRegistryBuilder
-from notionary.models.notion_database_response import NotionPageResponse
-from notionary.models.notion_page_response import DatabaseParent
+from notionary.blocks.client import NotionBlockClient
+from notionary.comments import CommentClient, Comment
+from notionary.blocks.syntax_prompt_builder import SyntaxPromptBuilder
+from notionary.blocks.models import DatabaseParent
+from notionary.blocks.registry.block_registry import BlockRegistry
+from notionary.database.client import NotionDatabaseClient
+from notionary.file_upload.client import NotionFileUploadClient
+from notionary.blocks.markdown.markdown_builder import MarkdownBuilder
+from notionary.schemas import NotionContentSchema
+from notionary.page import page_context
 from notionary.page.client import NotionPageClient
-from notionary.page.content.page_content_retriever import PageContentRetriever
-
-
-from notionary.page.content.page_content_writer import PageContentWriter
+from notionary.page.models import NotionPageResponse
+from notionary.page.page_content_deleting_service import PageContentDeletingService
+from notionary.page.page_content_writer import PageContentWriter
+from notionary.page.page_context import PageContextProvider, page_context
 from notionary.page.property_formatter import NotionPropertyFormatter
+from notionary.page.reader.page_content_retriever import PageContentRetriever
 from notionary.page.utils import extract_property_value
-
-from notionary.util import LoggingMixin, format_uuid, FuzzyMatcher, factory_only
-
+from notionary.util import LoggingMixin, extract_uuid, factory_only, format_uuid
+from notionary.util.fuzzy import find_best_match
 
 if TYPE_CHECKING:
     from notionary import NotionDatabase
@@ -26,14 +34,16 @@ class NotionPage(LoggingMixin):
     Managing content and metadata of a Notion page.
     """
 
-    @factory_only("from_page_id", "from_page_name")
+    @factory_only("from_page_id", "from_page_name", "from_url")
     def __init__(
         self,
         page_id: str,
         title: str,
         url: str,
+        archived: bool,
+        in_trash: bool,
         emoji_icon: Optional[str] = None,
-        properties: Optional[Dict[str, Any]] = None,
+        properties: Optional[dict[str, Any]] = None,
         parent_database: Optional[NotionDatabase] = None,
         token: Optional[str] = None,
     ):
@@ -43,36 +53,41 @@ class NotionPage(LoggingMixin):
         self._page_id = page_id
         self._title = title
         self._url = url
+        self._is_archived = archived
+        self._is_in_trash = in_trash
         self._emoji_icon = emoji_icon
         self._properties = properties
         self._parent_database = parent_database
 
         self._client = NotionPageClient(token=token)
+        self._block_client = NotionBlockClient(token=token)
+        self._comment_client = CommentClient(token=token)
         self._page_data = None
 
-        self._block_element_registry = BlockRegistryBuilder.create_full_registry()
+        self.block_element_registry = BlockRegistry.create_registry()
 
         self._page_content_writer = PageContentWriter(
             page_id=self._page_id,
-            client=self._client,
-            block_registry=self._block_element_registry,
+            block_registry=self.block_element_registry,
+        )
+
+        self._page_content_deleting_service = PageContentDeletingService(
+            page_id=self._page_id,
+            block_registry=self.block_element_registry,
         )
 
         self._page_content_retriever = PageContentRetriever(
-            page_id=self._page_id,
-            block_registry=self._block_element_registry,
+            block_registry=self.block_element_registry,
         )
 
+        self.page_context_provider = self._setup_page_context_provider()
+
     @classmethod
     async def from_page_id(
         cls, page_id: str, token: Optional[str] = None
     ) -> NotionPage:
         """
         Create a NotionPage from a page ID.
-
-        Args:
-            page_id: The ID of the Notion page
-            token: Optional Notion API token (uses environment variable if not provided)
         """
         formatted_id = format_uuid(page_id) or page_id
 
@@ -93,15 +108,15 @@ class NotionPage(LoggingMixin):
         workspace = NotionWorkspace()
 
         try:
-            search_results: List[NotionPage] = await workspace.search_pages(
-                page_name, limit=10
+            search_results: list[NotionPage] = await workspace.search_pages(
+                page_name, limit=5
             )
 
             if not search_results:
                 cls.logger.warning("No pages found for name: %s", page_name)
                 raise ValueError(f"No pages found for name: {page_name}")
 
-            best_match = FuzzyMatcher.find_best_match(
+            best_match = find_best_match(
                 query=page_name,
                 items=search_results,
                 text_extractor=lambda page: page.title,
@@ -131,6 +146,26 @@ class NotionPage(LoggingMixin):
             cls.logger.error("Error finding page by name: %s", str(e))
             raise
 
+    @classmethod
+    async def from_url(cls, url: str, token: Optional[str] = None) -> NotionPage:
+        """
+        Create a NotionPage from a Notion page URL.
+        """
+        try:
+            page_id = extract_uuid(url)
+            if not page_id:
+                raise ValueError(f"Could not extract page ID from URL: {url}")
+
+            formatted_id = format_uuid(page_id) or page_id
+
+            async with NotionPageClient(token=token) as client:
+                page_response = await client.get_page(formatted_id)
+                return await cls._create_from_response(page_response, token)
+
+        except Exception as e:
+            cls.logger.error("Error creating page from URL '%s': %s", url, str(e))
+            raise
+
     @property
     def id(self) -> str:
         """
@@ -161,30 +196,62 @@ class NotionPage(LoggingMixin):
         return self._emoji_icon
 
     @property
-    def properties(self) -> Optional[Dict[str, Any]]:
+    def properties(self) -> Optional[dict[str, Any]]:
         """
         Get the properties of the page.
         """
         return self._properties
 
     @property
-    def block_registry(self) -> BlockRegistry:
+    def is_archived(self) -> bool:
+        return self._is_archived
+
+    @property
+    def is_in_trash(self) -> bool:
+        return self._is_in_trash
+
+    def get_prompt_information(self) -> str:
+        markdown_syntax_builder = SyntaxPromptBuilder()
+        return markdown_syntax_builder.build_concise_reference()
+
+    async def get_comments(self) -> list[Comment]:
+        return await self._comment_client.list_all_comments_for_page(
+            page_id=self._page_id
+        )
+
+    async def post_comment(
+        self,
+        content: str,
+        *,
+        discussion_id: Optional[str] = None,
+        rich_text: Optional[list[dict[str, Any]]] = None,
+    ) -> Optional[Comment]:
         """
-        Get the block element registry associated with this page.
+        Post a comment on this page.
+
+        Args:
+            content: The plain text content of the comment
+            discussion_id: Optional discussion ID to reply to an existing discussion
+            rich_text: Optional rich text formatting for the comment content
 
         Returns:
-            BlockElementRegistry: The registry of block elements.
+            Comment: The created comment object, or None if creation failed
         """
-        return self._block_element_registry
-
-    def get_notion_markdown_system_prompt(self) -> str:
-        """
-        Get the formatting prompt for the page content manager.
-
-        Returns:
-            str: The formatting prompt.
-        """
-        return self._block_element_registry.get_notion_markdown_syntax_prompt()
+        try:
+            # Use the comment client to create the comment
+            comment = await self._comment_client.create_comment(
+                page_id=self._page_id,
+                content=content,
+                discussion_id=discussion_id,
+                rich_text=rich_text,
+            )
+            self.logger.info(f"Successfully posted comment on page '{self._title}'")
+            return comment
+        except Exception as e:
+            self.logger.error(
+                f"Failed to post comment on page '{self._title}': {str(e)}"
+            )
+            return None
 
     async def set_title(self, title: str) -> str:
         """
@@ -204,40 +271,52 @@ class NotionPage(LoggingMixin):
 
         except Exception as e:
             self.logger.error("Error setting page title: %s", str(e))
-            return None
 
-    async def append_markdown(self, markdown: str, append_divider=False) -> bool:
+    async def append_markdown(
+        self,
+        content: Union[
+            str, Callable[[MarkdownBuilder], MarkdownBuilder], NotionContentSchema
+        ],
+    ) -> bool:
         """
-        Append markdown content to the page.
+        Append markdown content to the page using text, builder callback, MarkdownDocumentModel, or NotionContentSchema.
         """
-        return await self._page_content_writer.append_markdown(
-            markdown_text=markdown, append_divider=append_divider
-        )
+        async with page_context(self.page_context_provider):
+            result = await self._page_content_writer.append_markdown(
+                content=content,
+            )
+            return result is not None
 
-    async def clear_page_content(self) -> bool:
-        """
-        Clear all content from the page.
-        """
-        return await self._page_content_writer.clear_page_content()
-
-    async def replace_content(self, markdown: str) -> bool:
+    async def replace_content(
+        self,
+        content: Union[
+            str, Callable[[MarkdownBuilder], MarkdownBuilder], NotionContentSchema
+        ],
+    ) -> bool:
         """
         Replace the entire page content with new markdown content.
 
         Args:
-            markdown: The new markdown content.
+            content: Either raw markdown text, a callback function that receives a MarkdownBuilder,
+                    a MarkdownDocumentModel, or a NotionContentSchema
 
         Returns:
-            str: Status or confirmation message.
+            bool: True if successful, False otherwise
         """
-        clear_result = await self._page_content_writer.clear_page_content()
+        clear_result = await self._page_content_deleting_service.clear_page_content()
         if not clear_result:
             self.logger.error("Failed to clear page content before replacement")
-            return False
 
-        return await self._page_content_writer.append_markdown(
-            markdown_text=markdown, append_divider=False
+        result = await self._page_content_writer.append_markdown(
+            content=content,
         )
+        return result is not None
+
+    async def clear_page_content(self) -> str:
+        """
+        Clear all content from the page.
+        """
+        return await self._page_content_deleting_service.clear_page_content()
 
     async def get_text_content(self) -> str:
         """
@@ -246,7 +325,10 @@ class NotionPage(LoggingMixin):
         Returns:
             str: The text content of the page.
         """
-        return await self._page_content_retriever.get_page_content()
+        blocks = await self._block_client.get_blocks_by_page_id_recursively(
+            page_id=self._page_id
+        )
+        return await self._page_content_retriever.convert_to_markdown(blocks=blocks)
 
     async def set_emoji_icon(self, emoji: str) -> Optional[str]:
         """
@@ -265,6 +347,32 @@ class NotionPage(LoggingMixin):
             self.logger.error(f"Error updating page emoji: {str(e)}")
             return None
 
+    async def create_child_database(self, title: str) -> NotionDatabase:
+        from notionary import NotionDatabase
+
+        database_client = NotionDatabaseClient(token=self._client.token)
+
+        create_database_response = await database_client.create_database(
+            title=title,
+            parent_page_id=self._page_id,
+        )
+
+        return await NotionDatabase.from_database_id(
+            id=create_database_response.id, token=self._client.token
+        )
+
+    async def create_child_page(self, title: str) -> NotionPage:
+        from notionary import NotionPage
+
+        child_page_response = await self._client.create_page(
+            parent_page_id=self._page_id,
+            title=title,
+        )
+
+        return await NotionPage.from_page_id(
+            page_id=child_page_response.id, token=self._client.token
+        )
+
     async def set_external_icon(self, url: str) -> Optional[str]:
         """
         Sets the page icon to an external image.
@@ -352,7 +460,7 @@ class NotionPage(LoggingMixin):
 
     async def _get_relation_property_values_by_name(
         self, property_name: str
-    ) -> List[str]:
+    ) -> list[str]:
         """
         Retrieve the titles of all related pages for a relation property.
         """
@@ -365,7 +473,7 @@ class NotionPage(LoggingMixin):
         ]
         return [page.title for page in notion_pages if page]
 
-    async def get_options_for_property_by_name(self, property_name: str) -> List[str]:
+    async def get_options_for_property_by_name(self, property_name: str) -> list[str]:
         """
         Get the available options for a property (select, multi_select, status, relation).
         """
@@ -386,7 +494,6 @@ class NotionPage(LoggingMixin):
             )
             return []
 
-    # Diese Methode hier sollte auch für relation properties funktionieren aber gerne auch eine dedizierte hier
     async def set_property_value_by_name(self, property_name: str, value: Any) -> Any:
         """
         Set the value of a specific property by its name.
@@ -425,8 +532,8 @@ class NotionPage(LoggingMixin):
             return None
 
     async def set_relation_property_values_by_name(
-        self, property_name: str, page_titles: List[str]
-    ) -> List[str]:
+        self, property_name: str, page_titles: list[str]
+    ) -> list[str]:
         """
         Add one or more relations to a relation property.
         """
@@ -493,7 +600,7 @@ class NotionPage(LoggingMixin):
         """
         Create NotionPage instance from API response.
         """
-        from notionary.database.notion_database import NotionDatabase
+        from notionary.database.database import NotionDatabase
 
         title = cls._extract_title(page_response)
         emoji = cls._extract_emoji(page_response)
@@ -510,6 +617,8 @@ class NotionPage(LoggingMixin):
             title=title,
             url=page_response.url,
             emoji_icon=emoji,
+            archived=page_response.archived,
+            in_trash=page_response.in_trash,
             properties=page_response.properties,
             parent_database=parent_database,
             token=token,
@@ -560,3 +669,10 @@ class NotionPage(LoggingMixin):
         parent = page_response.parent
         if isinstance(parent, DatabaseParent):
             return parent.database_id
+
+    def _setup_page_context_provider(self) -> PageContextProvider:
+        return PageContextProvider(
+            page_id=self._page_id,
+            database_client=NotionDatabaseClient(token=self._client.token),
+            file_upload_client=NotionFileUploadClient(),
+        )
