@@ -1,15 +1,14 @@
+from .factory_only import factory_only
 from .logging_mixin import LoggingMixin
-from .singleton_decorator import singleton
-from .page_id_utils import format_uuid
-from .fuzzy_matcher import FuzzyMatcher
-from .factory_decorator import factory_only
+from .page_id_utils import extract_uuid, format_uuid
+from .singleton import singleton
 from .singleton_metaclass import SingletonMetaClass
 
 __all__ = [
     "LoggingMixin",
-    "singleton_decorator",
+    "singleton",
     "format_uuid",
-    "FuzzyMatcher",
+    "extract_uuid",
     "factory_only",
     "singleton",
     "SingletonMetaClass",
